﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.NetworkInformation;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

/*
 *         Progetto: Chat P2P basata su comunicazioni UDP e TCP
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 3/11/2015
 *  
 *      Descrizione: Routine di gestione della finestra per l'inserimento dell'utente.
 *                   Consente la selezione dell'interfaccia di rete.    
 *                   
 */
namespace ChatWPF
{
    class InfoIndirizzo
    {
        public IPAddress IndirizzoIP { get; set; }
        public IPAddress NetMask { get; set; }
        public IPAddress Broadcast { get; set; }
        public long ByteRicevuti { get; set; }

        public override string ToString()
        {
            return String.Format("{0} (Netmask {1}; broadcast {2})", IndirizzoIP, NetMask, Broadcast);
        }

    }
    /// <summary>
    /// Logica di interazione per UtenteDialog.xaml
    /// </summary>
    public partial class UtenteDialog : Window
    {
        public string NomeUtente
        {
            get
            {
                return tbNomeUtente.Text.Trim();
            }
        }

        public IPAddress IndirizzoIp
        {
            get
            {
                return (cbIndirizzi.SelectedIndex < 0 ? null : ((InfoIndirizzo)cbIndirizzi.SelectedValue).IndirizzoIP);
            }
        }

        public IPAddress Broadcast
        {
            get
            {
                return (cbIndirizzi.SelectedIndex < 0 ? null : ((InfoIndirizzo)cbIndirizzi.SelectedValue).Broadcast);
            }
        }


        public UtenteDialog()
        {
            InitializeComponent();
        }

        private InfoIndirizzo[] IndirizziSchede()
        {
            List<InfoIndirizzo> lista = new List<InfoIndirizzo>();

            foreach (NetworkInterface nic in NetworkInterface.GetAllNetworkInterfaces())
                if (nic.NetworkInterfaceType == NetworkInterfaceType.Wireless80211 || nic.NetworkInterfaceType == NetworkInterfaceType.Ethernet)
                    foreach (UnicastIPAddressInformation ind in nic.GetIPProperties().UnicastAddresses)
                        if (ind.Address.AddressFamily == System.Net.Sockets.AddressFamily.InterNetwork)
                        {
                            IPAddress ipa = ind.Address;
                            IPAddress mask = ind.IPv4Mask;

                            // Determina l'indirizzo di broadcast
                            UInt32 ipa32 = BitConverter.ToUInt32(ipa.GetAddressBytes(), 0);
                            UInt32 mask32 = BitConverter.ToUInt32(mask.GetAddressBytes(), 0);
                            UInt32 bcast32 = (UInt32)(ipa32 | ~mask32);

                            IPAddress bcast = new IPAddress(bcast32);
                            long ric = nic.GetIPv4Statistics().BytesReceived;

                            lista.Add(new InfoIndirizzo() {IndirizzoIP = ipa, NetMask = mask, Broadcast = bcast, ByteRicevuti = ric});
                        }

            var risultato = from elemento in lista
                                orderby elemento.ByteRicevuti descending
                                select elemento;

            return risultato.ToArray();

        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            DialogResult = true;
        }

        private void wndDatiUtente_Loaded(object sender, RoutedEventArgs e)
        {
            foreach (InfoIndirizzo i in IndirizziSchede())
                cbIndirizzi.Items.Add(i);

            if (cbIndirizzi.Items.Count > 0)
                cbIndirizzi.SelectedIndex = 0;
        }

        private void tbNomeUtente_TextChanged(object sender, TextChangedEventArgs e)
        {
            btnOk.IsEnabled = tbNomeUtente.Text.Trim().Length > 0;
        }
    }
}
