﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;

namespace ChatWPF
{
    public static class Utility
    {
        const int portaServer = 4200;


        public static TcpListener ListTcp {get; set;}
        public static UdpClient ListUdp { get; set; }
        public static string UtenteLocale { get; set; }
        public static IPAddress IndirizzoLocale { get; set; }
        public static IPAddress Broadcast { get; set; }

        public static bool InizializzaListener() {

            bool errore = false;
            try
            {
                ListUdp = new UdpClient(portaServer);
            }
            catch (Exception)
            {
                MessageBox.Show("Impossibile avviare la comunicazione UDP. " +
                                "Verificare che non siano attive altre istanze del programma.",
                                "Errore fatale",
                                MessageBoxButton.OK,
                                MessageBoxImage.Stop);

                errore = true;
            }

            if (!errore)
            {
                try
                {
                    ListTcp = new TcpListener(IndirizzoLocale, portaServer);
                    ListTcp.Start(30);
                }
                catch (Exception)
                {
                    MessageBox.Show("Impossibile avviare la comunicazione TCP. " +
                                    "Verificare che non siano attive altre istanze del programma.",
                                    "Errore fatale",
                                    MessageBoxButton.OK,
                                    MessageBoxImage.Stop);

                    errore = true;
                }
            }

            if (errore)
            {
                if (ListUdp != null)
                {
                    ListUdp.Close();
                    ListUdp = null;
                }
                if (ListTcp != null)
                {
                    ListTcp.Stop();
                    ListTcp = null;
                }
            }

            return !errore;
        }

        public static async void AvviaListenerTcp(Action<IPEndPoint, string> azione)
        {
            bool finito = false;

            while (!finito)
            {
                try
                {
                    TcpClient cl = await ListTcp.AcceptTcpClientAsync();

                    using (NetworkStream networkStream = cl.GetStream())
                    {
                        byte[] buffer = new byte[4096];
                        int offset = 0;
                        int byteLetti;
                        do
                        {
                            byteLetti = await networkStream.ReadAsync(buffer, offset, buffer.Length - offset);
                            offset += byteLetti;
                            Thread.Sleep(100);
                        } while (byteLetti > 0);

                        IPEndPoint ep = (IPEndPoint)cl.Client.RemoteEndPoint;
                        string m = Encoding.UTF8.GetString(buffer, 0, offset);

                        azione(ep, m);
                    }
                }
                catch (ObjectDisposedException)
                {
                    finito = true;
                }
                catch (SocketException e)
                {
                    MessageBox.Show(
                        e.Message,
                        "Errore durante la ricezione dei dati TCP",
                        MessageBoxButton.OK,
                        MessageBoxImage.Error
                    );

                }

            }
        }

        public static async void AvviaListenerUdp(Action<IPEndPoint, string> azione)
        {
            bool finito = false;

            while (!finito)
            {
                try
                {
                    UdpReceiveResult risultato = await ListUdp.ReceiveAsync();
                    IPEndPoint ep = risultato.RemoteEndPoint;
                    string msg = Encoding.UTF8.GetString(risultato.Buffer);

                    azione(ep, msg);

                }
                catch (ObjectDisposedException)
                {
                    finito = true;
                }
            }
        }

        public static void InviaMessaggioUdp(IPAddress destinatario, string messaggio)
        {

            try
            {
                using (UdpClient mittente = new UdpClient())
                {
                    byte[] buffer = Encoding.UTF8.GetBytes(messaggio);
                    mittente.Send(buffer, buffer.Length, new IPEndPoint(destinatario, portaServer));
                }

            }
            catch (Exception e) {
                MessageBox.Show(
                    e.Message,
                    "Errore nell'invio dei dati UDP",
                    MessageBoxButton.OK,
                    MessageBoxImage.Error
                );
            }
        }

        public static async void InviaMessaggioTcpAsync(IPAddress destinatario, string messaggio)
        {
            try
            {
                using (var client = new TcpClient())
                {
                    await client.ConnectAsync(destinatario, portaServer);
                    using (NetworkStream ns = client.GetStream())
                    {
                        byte[] buffer = Encoding.UTF8.GetBytes(messaggio);

                        await ns.WriteAsync(buffer, 0, buffer.Length);
                    }
                }
            }
            catch (Exception e)
            {
                MessageBox.Show(
                    e.Message,
                    "Errore nell'invio dei dati TCP",
                    MessageBoxButton.OK,
                    MessageBoxImage.Error
                );
            }
        }

    }

}
